; *************************************************************
function wmoment, x, w 
; *************************************************************
; Function to compute average and variance of
; a field x weighted by a field of weights w

tweight=total(w, /double)
if tweight le 0.0 then begin
  print, "Sum of weights = 0. No valid data found. Aborting."
  return, [0.0d0,0.0d0,0.0d0]
endif

; First create zero average for shorter equations
av=total(x*w,/double)/tweight
x=x-av
; Variance
squ=total(x*x*w,/double)/tweight
var=squ
; Bensch error
err=sqrt(total(x^4*w,/double)/tweight-squ*squ)
return, [av,var,err]
end

; *************************************************************
function corefilt, x, d 
; *************************************************************
; Function providing the FT of the French hat core

; Avoid division by zero
x=x+1.0d-7

y=2.0d0*beselj(x,1,/double)/x
return, y
end

; *************************************************************
function gausscorefilt, x, d 
; *************************************************************
; Function providing the FT of the Gauss core

xx=0.5d0*x
y=exp(-xx^2)
return, y
end

; *************************************************************
function annulfilt, x, d
; *************************************************************
; Function providing the FT of the French hat annulus

; Avoid division by zero
x=x+1.0d-7
c=double(d)

x3=c*x
y=c^2*beselj(x3,1,/double)/x3-beselj(x,1,/double)/x
y=y*2.0d0/(c^2-1.0d0)
return, y
end

; *************************************************************
function gaussannulfilt, x, d
; *************************************************************
; Function providing the FT of the Gauss filter annulus

c=double(d)

xx=0.5d0*x
x3=c*xx
y=c^2*exp(-x3^2)-exp(-xx^2)
y=y/(c^2-1.0d0)
return, y
end

; *************************************************************
function delta_weighted, rdat, weight, pixel=pixel, drad=v, $
	cut=cut, filter=filter, signifexp=signifexp
; *************************************************************
; 2-D Allan analysis
; Input parameter:   rdat = data field
;                    drad = radienverhaeltnis des besselfilters
;                    filter = 0 for French hat
;                             1 for Mexican hat
; Output parameter:  delta2d = field of lags in first column
;		               Delta variances in second column
;
; Tolerance for numerical FT
tol=1.0d-3

; Field size
a=size(rdat)
wa=size(weight)

; Consistency checks
if total(a ne wa) then begin
  print, "Size mismatch between data field and weighing function. Aborting."
  return, [[0.0d0],[0.0d0],[0.0d0]]
endif
valid=where(weight gt 0.0d0,count)
if count lt 1 then begin
  print, "Weights are everywhere zero. Aborting."
  return, [[0.0d0],[0.0d0],[0.0d0]]
endif
nx=a[1]
ny=a[2]
qrat=(double(ny)/nx)^2

; Pixel size
if n_elements(pixel) eq 0 then pixel=[1.0d0,1.0d0] $
 else if n_elements(pixel) eq 1 then pixel=[pixel,pixel] 
xqpixel=(1.0d0/pixel[0])^2
yqpixel=(1.0d0/pixel[1])^2

; Numerical control parameters
if not keyword_set(cut) then cut=1.0d0
if n_elements(signifexp) eq 0 then signifexp=1.0d0
if (signifexp lt 1.0d-3) then signifexp=1.0d-3

; Filter name
if not keyword_set(filter) then begin
; French hat
		core="corefilt"
		annu="annulfilt"
		aa=0.5169d0
	        bb=0.5723d0
endif else begin
; Mexican hat
		core="gausscorefilt"
		annu="gaussannulfilt"
		aa=0.9192d0
		bb=0.8215d0
		; aa=0.8875 Melanie's numbers
		; bb=0.8323
		cut=cut*1.5d0
endelse

; Numerical control parameters
if not keyword_set(v) then v=3.0d0
drad=aa+bb*v

; Treat data field
data=dcomplexarr(nx,ny)
wdata=dcomplexarr(nx,ny)
; Fourier transform  and square
fwdfourier, rdat*weight,data
fwdfourier, weight,wdata

; Now calculate Delta variance

; First initialize constants
wgt=double(nx*ny)
da=(2.0d0/drad > 1.41d0)*min(pixel)
db=min([nx*pixel[0],ny*pixel[1]])/(cut*v)
damp=10.0d0
nl=fix(damp*alog((db/da -1.0d0)/damp+1.0d0))
allan=dblarr(nl,3)

; Make index fields
xindex=nx/2-findgen(nx)
yindex=make_array(ny, value=1.0d0)
xfeld=xindex#yindex
xfeld=xfeld*xfeld
xindex=make_array(nx, value=1.0d0)
yindex=ny/2-findgen(ny)
yfeld=xindex#yindex
yfeld=yfeld*yfeld

; Loop for lags
for jl=0,nl-1 do begin
  d=da*(damp*(exp(jl/damp)-1.0d0)+1.0d0)
  
; Create filter map
  pid=!dpi*d/ny
  cfilter=wgt*call_function(core,pid*sqrt(xfeld*qrat*xqpixel+yfeld*yqpixel),v)
  afilter=wgt*call_function(annu,pid*sqrt(xfeld*qrat*xqpixel+yfeld*yqpixel),v)

; Multiply in Fourier space and backtransform
  bckfourier, wdata*cfilter, wcore
  bckfourier, wdata*afilter, wannul
  bckfourier, data*cfilter, dcore
  bckfourier, data*afilter, dannul

; Combine, weigh by product of weights
; Different definitions of weightfeld should be tried
  weightfeld=(abs(wcore*wannul) - tol) > 0.0d0
  filteredfeld=weightfeld
  valid=where(weightfeld gt 0.0d0)
  weightfeld[valid]=weightfeld[valid]+tol
  filteredfeld[valid]=dcore[valid]/wcore[valid]-dannul[valid]/wannul[valid]

; Compute new Delta variance
  statistics=wmoment(filteredfeld,weightfeld)
  allan[jl,0]=d*drad/2.0d0
  allan[jl,1]=statistics[1]
; Number of independent settings
; Check exact factor
  nindep=total(weightfeld^signifexp)*pixel[0]*pixel[1]/allan[jl,0]^2
; Bensch error
  allan[jl,2]=statistics[2]/sqrt(nindep)
; Control output on error
; print, statistics[2]/statistics[1], nindep, allan[jl,2]
; Error as Poisson-error
; allan[jl,2]=statistics[1]/sqrt(nindep)
print, jl, ' Scale: ', allan[jl,0], ' D-Var: ', allan[jl,1], '+/-', allan[jl,2]
endfor

return, allan
end
