pro overlay, a, xx, yy, NLEVELS = nlev, levels=levels, TITLES = titles, $
                        NOASPECT= noaspect, WINDOW_SCALE = window_scale, $
			GRID = grid, radec=radec 
;+
; NAME:
;	OVERLAY
;
; PURPOSE:
;	Overlay an image and a contour plot.
;
; CATEGORY:
;	General graphics.
;
; CALLING SEQUENCE:
;	OVERLAY, A [, XX, YY]
;
; INPUTS:
;	A:	The two-dimensional array to display.
;	XX,YY:	The fields containing the axis grids.
;
; KEYWORD PARAMETERS:
; WINDOW_SCALE:	Set this keyword to scale the window size to the image size.
;		Otherwise, the image size is scaled to the window size.
;		This keyword is ignored when outputting to devices with 
;		scalable pixels (e.g., PostScript).
;
; OUTPUTS:
;	No explicit outputs.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;	The currently selected display is affected.
;
; RESTRICTIONS:
;	None.
;
; PROCEDURE:
;	If the device has scalable pixels, then the image is written over
;	the plot window.
;
; MODIFICATION HISTORY:
;	DMS, May, 1988.
;-

;on_error,2                      ;Return to caller if an error occurs
sz = size(a)			;Size of image
if sz[0] lt 2 then message, 'Parameter not 2D'

	;set window used by contour
contour,[[0,0],[1,1]],/nodata, xstyle=4, ystyle = 4

; Obtain window settings
px = !x.window * !d.x_vsize	;Get size of window in device units
py = !y.window * !d.y_vsize
swx = (px[1]-px[0])*0.875	;Size in x in device units, add space for color bar
swy = py[1]-py[0]		;Size in Y
six = float(sz[1])		;Image sizes
siy = float(sz[2])
aspi = six / siy		;Image aspect ratio
aspw = swx / swy		;Window aspect ratio
f = aspi / aspw			;Ratio of aspect ratios

; exception handling for NaNs in the data
valid=where(FINITE(a), count)
if count lt 1 then message, "No valid data points found."
validpart=a[valid]
maxval=max(validpart)
minval=min(validpart)
epsilon=1.0/256.0
b=make_array(size=size(a),value=minval-epsilon*(maxval-minval))
b[valid]=validpart

; Produce color plot
if (!d.flags and 1) ne 0 then begin	;Scalable pixels?
				;Adjust window size
  if not keyword_set(noaspect) then begin
    if f ge 1.0 then swy = swy / f else swx = swx * f
  endif

  tv,bytscl(b,min=minval,max=maxval),px[0],py[0],$
			xsize = swx, ysize = swy, /device

endif else begin	;Not scalable pixels	
   if keyword_set(window_scale) then begin ;Scale window to image?
	tv,bytscl(b,min=minval,max=maxval),px[0],py[0]	;Output image
	swx = six		;Set window size from image
	swy = siy
    endif else begin		;Scale window
        if not keyword_set(noaspect) then begin
          if f ge 1.0 then swy = swy / f else swx = swx * f
        endif
	tv,congrid(bytscl(b,min=minval,max=maxval),swx,swy),px[0],py[0]
	endelse			;window_scale
     endelse			;scalable pixels

mx = !d.n_colors-1		;Brightest color
				;color vectors

; Contour level definition
; default
nlevbas=n_elements(levels)
if n_elements(nlev) EQ 0 then begin
	nlev=nlevbas
	if nlev eq 0 then nlev=5
endif

; No contours for constant surface
if maxval eq minval then nlev=0

if nlev NE 0 then begin
  colors=intarr(nlev)
  for i=0,(nlev-1)/2 do colors[i]=mx
  for i=(nlev-1)/2+1,nlev-1 do colors[i]=0
  nodata=0
; Create really equidistant levels
  if nlevbas eq 0 then begin
    a0=minval
    da=(maxval-minval)/(nlev+1)
    levels=fltarr(nlev)
    for j=0,nlev-1 do levels[j]=a0+(j+1)*da
  endif else begin
    levels=levels[0:nlev-1]
  endelse
; No contours
endif else begin
  colors=[mx]
  nodata=1
  levels=[minval]
endelse

; Create the color bar
steps = 100
xsize = swx/10
ysize = swy/steps
xpos = px[0] + swx +1
ypos = py[0]

a0=minval
da=(maxval-minval)/(steps-1)
feld2=fltarr(2,steps)
for j=0,steps-1 do begin feld2[0,j]=a0+j*da & feld2[1,j]=feld2[0,j] & endfor

if (!d.flags and 1) ne 0 then begin	;Scalable pixels
  tvscl,feld2,xpos,ypos,xsize = xsize, ysize = swy, /device
endif else begin			;Not scalable pixels	
for j=0,steps-1 do begin
  color=fix((float(!d.table_size)/steps)*j)
  tv, replicate(color,xsize,fix(ysize)+1), fix(xpos), fix(ypos+j*ysize)
endfor
endelse

ax1=[0.0,1.0]
ax2=fltarr(steps)
x0=minval
dx=(maxval-minval)/steps
for j=0,steps-1 do ax2[j]=x0+(j+0.5)*dx
contour, feld2, ax1,ax2, /noerase, $
         pos=[xpos,ypos,fix(xpos+xsize),ypos+swy], /dev, $
         levels = levels, c_color = colors, nodata = nodata, ystyle=5, $
         xticks=1, xtickname=replicate(' ',2), xtickformat='(A1)', xminor=1 

if n_elements(titles) EQ 0 then titles=[' ',' ',' ']

axis, yaxis=1, ytitle=titles[2], ystyle=1, ticklen=0.2, yminor=5

; Now add the contour plot
nx=n_elements(xx)-1
ny=n_elements(yy)-1
if ((nx lt 1) or (ny lt 1)) then begin
	   contour,a,/noerase,/xst,/yst,$	
	   pos = [px[0],py[0], px[0]+swx,py[0]+swy],/dev,$
	   levels = levels, c_color =  colors, nodata = nodata, $
           xtitle=titles[0], ytitle=titles[1]
endif else begin
  if keyword_set(radec) then begin
	   ticks=radeclabels(xx,yy)
	   xx1=xx[0]-(xx[1]-xx[0])*0.5
	   xx2=xx[nx]+(xx[nx]-xx[nx-1])*0.5
	   yy1=yy[0]-(yy[1]-yy[0])*0.5
	   yy2=yy[ny]+(yy[ny]-yy[ny-1])*0.5
	   contour,a,xx,yy,/noerase,/xst,/yst,$
	   xrange=[xx1,xx2], yrange=[yy1,yy2],$
	   pos = [px[0],py[0], px[0]+swx,py[0]+swy],/dev,$
	   levels = levels, c_color =  colors, nodata = nodata, $
           xtitle=titles[0], ytitle=titles[1], $
           xticks=ticks.xTicks, yticks=ticks.yTicks, xtickv=ticks.xPos, $
	   ytickv=ticks.yPos, xtickname=ticks.xNames, ytickname=ticks.yNames
  endif else begin
	   xx1=xx[0]-(xx[1]-xx[0])*0.5
	   xx2=xx[nx]+(xx[nx]-xx[nx-1])*0.5
	   yy1=yy[0]-(yy[1]-yy[0])*0.5
	   yy2=yy[ny]+(yy[ny]-yy[ny-1])*0.5
	   contour,a,xx,yy,/noerase,/xst,/yst,$
	   xrange=[xx1,xx2], yrange=[yy1,yy2],$
	   pos = [px[0],py[0], px[0]+swx,py[0]+swy],/dev,$
	   levels = levels, c_color =  colors, nodata = nodata, $
           xtitle=titles[0], ytitle=titles[1]
  endelse
endelse

; Title
xchar=!d.x_ch_size*(1.0+!p.charsize)
ychar=!d.y_ch_size*(1.0+!p.charsize)
if (n_elements(titles) gt 3) then xyouts, px[0]+2.0*xchar, py[0]+swy-1.0*ychar, /dev, titles[3], $
	   charsize=!p.charsize*1.2, color=colors[0], charthick=2.0

; Possible return grid
ax=px[0]
ay=py[0]
grid=[ax,swx,nx,ay,swy,ny]

return
end
